"""Glotarans fitmodel package"""

from . import (matrix, matrix_group_generator, fitmodel, result)

FitModel = fitmodel.FitModel
MatrixGroupGenerator = matrix_group_generator.MatrixGroupGenerator
Matrix = matrix.Matrix
Result = result.Result
