"""Glotarans dataIO package"""

from . import wavelength_time_explicit_file

WavelengthExplicitFile = wavelength_time_explicit_file.WavelengthExplicitFile
TimeExplicitFile = wavelength_time_explicit_file.TimeExplicitFile
